package com.persagy.adm.server.custom.service.impl.offline_data_download;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.adm.server.custom.common.AdmConst;
import com.persagy.adm.server.custom.common.AdmRequest;
import com.persagy.adm.server.custom.entity.BuildingData;
import com.persagy.adm.server.custom.entity.Pagination;
import com.persagy.adm.server.custom.service.ServiceUtil;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.digital.client.DigitalObjectClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 离线数据下载-对象实例数据
 */
@Component
@Slf4j
public class FillObject implements FillData<BuildingData>{
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private DigitalObjectClient objectClient;
    @Autowired
    DownLoadDataDeal downLoadDataDeal;

    @Override
    public void fill(AdmRequest admRequest, BuildingData buildingData) {
        log.info("填充对象实例纸数据开始");
        //从bdtp下载对象（空间、竖井、系统、设备、元空间)
        QueryCriteria criteria = ServiceUtil.getQueryCriteria(objectMapper, AdmConst.OBJ_TYPE_SPACE, AdmConst.OBJ_TYPE_SHAFT, AdmConst.OBJ_TYPE_SYSTEM, AdmConst.OBJ_TYPE_EQUIPMENT, AdmConst.OBJ_TYPE_ISPACE);
        String operator = CriteriaUtils.getOperator(admRequest.getClientId(), admRequest.getUserId());
        CriteriaUtils.buildDownloadQueryCriteria(criteria,operator,admRequest.getBdtpDownloadTs());

        // 增量数据下载
        if (admRequest.getBdtpDownloadTs() != null && admRequest.getBdtpDownloadTs() > 0){
            criteria.getCriteria().putObject("ts").put("$gt", admRequest.getBdtpDownloadTs());
            criteria.getCriteria().putObject("modifier").put("$ne", operator);
        }
        criteria.getCriteria().put("buildingId", admRequest.getBuildingId());
        List<ObjectNode> objs = ServiceUtil.queryAllPage(() -> objectClient.query(admRequest.getGroupCode(), admRequest.getProjectId(), AdmConst.APP_ID, admRequest.getUserId(), criteria), criteria, new Pagination(500));
        if (objs == null)
            objs = new ArrayList<>(0);

        // 处理下载数据并压缩公共字段
        downLoadDataDeal.packInfos(objs);
        // 填充ts
        downLoadDataDeal.test4Ts(objs);
        buildingData.setObjects(objs);
        log.info("填充对象实例数据结束");
    }
}
