package com.persagy.adm.steward.client;

import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.adm.steward.basic.config.StewardFeignConfig;
import com.persagy.adm.steward.basic.model.response.BossResponse;
import com.persagy.adm.steward.entity.ProductFinishNumReq;
import com.persagy.dmp.common.model.response.CommonResult;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;
import java.util.Set;

/***
 * Description:对象管理Client
 * @author : lijie
 * @date :2021/8/19 17:38
 * Update By lijie 2021/8/19 17:38
 */
@FeignClient(name = "boss-steward", configuration = StewardFeignConfig.class,
        url = "${persagy.boss.steward.service:}",path = "/")
public interface BossStewardClient {

    /**
     * 根据项目id批量查询项目信息
     * @param projectIds: 项目id数组
     * @return 对象数组
     */
    @PostMapping("test")
    BossResponse<Void> test(@RequestHeader(value = "encryption-rsa", required = false,defaultValue = "true") Boolean rsaFlag,
                            @RequestBody Set<String> projectIds);

    /**
     * 项目交付完成接口回调
     * @param productFinishNumReqList: 交付完成信息
     * @return 对象数组
     */
    @PostMapping("test")
    BossResponse<List<ProductFinishNumReq>> finishDeliverPlan(@RequestHeader(value = "encryption-rsa", required = false,defaultValue = "true") Boolean rsaFlag,
                                                              @RequestBody List<ProductFinishNumReq> productFinishNumReqList);
}
